import json
import os
import time
import uuid

class BackupInfo:
    DEFAULT_TIMEOUT = 5
    DEFAULT_PORT = 22
    BACKUP_FOLDER = "backups"
    
    def __init__(self, backup_folder=BACKUP_FOLDER):
        self.backups = {}
        self.backup_folder = backup_folder
        os.makedirs(self.backup_folder, exist_ok=True)
        self.load_backups_from_files()
        self.check_and_fill_missing_fields()

    def add_backup(self, backup_details):
        # Validate backup details
        required_keys = ['name', 'source_folder', 'destination_folder', 'hostname', 'username']
        for key in required_keys:
            if key not in backup_details or not backup_details[key]:
                raise ValueError(f"Missing required backup detail: {key}")

        # Generate UUID for the backup    
        backup_uuid = str(uuid.uuid4())

        # Create backup dictionary
        backup = {
            'uuid': backup_uuid,
            'name': backup_details.get('name'),
            'source_folder': backup_details.get('source_folder'),
            'destination_folder': backup_details.get('destination_folder'),
            'hostname': backup_details.get('hostname'),
            'username': backup_details.get('username'),
            'port': self.DEFAULT_PORT,
            'key_path': backup_details.get('key_path'),
            'key_passphrase': backup_details.get('key_passphrase'),
            'timeout': backup_details.get('timeout', self.DEFAULT_TIMEOUT),
            'created_at': time.time(),
            'updated_at': time.time(),
        }
       
        # Save the backup
        self.save_backup_to_file(backup)
        self.backups[backup_uuid] = backup
        print(f"Backup '{backup['name']}' saved successfully.")
        return backup

    def save_backup_to_file(self, backup):
        filename = os.path.join(self.backup_folder, f"{backup['uuid']}.json")
        try:
            with open(filename, 'w') as file:
                json.dump(backup, file, indent=4)
        except IOError as e:
            print(f"An error occurred while saving the backup: {e}")

    def load_backups_from_files(self):
        loaded_backups = {}
        for filename in os.listdir(self.backup_folder):
            if filename.endswith(".json"):
                file_path = os.path.join(self.backup_folder, filename)
                try:
                    with open(file_path, 'r') as file:
                        backup = json.load(file)
                    loaded_backups[backup['uuid']] = backup
                except (IOError, json.JSONDecodeError) as e:
                    print(f"An error occurred while loading the backup from {filename}: {e}")
        self.backups = loaded_backups
        return loaded_backups

    def delete_backup(self, backup_uuid):
        filename = os.path.join(self.backup_folder, f"{backup_uuid}.json")
        try:
            if backup_uuid not in self.backups:
                raise KeyError(f"Backup with UUID '{backup_uuid}' not found in the backup list.")
            
            os.remove(filename)
            del self.backups[backup_uuid]
            print(f"Backup with UUID '{backup_uuid}' deleted successfully.")
        except FileNotFoundError:
            print(f"Backup file with UUID '{backup_uuid}' not found.")
        except KeyError as e:
            print(f"Error deleting backup: {e}")
        except Exception as e:
            print(f"An unexpected error occurred while deleting the backup with UUID '{backup_uuid}': {e}")

    def update_backup(self, backup_uuid, print_message = True, **kwargs):
        if backup_uuid not in self.backups:
            if print_message:
                print(f"Backup with UUID '{backup_uuid}' not found.")
            return
        
        backup = self.backups[backup_uuid]
        updated = False

        for key, value in kwargs.items():
            if key == 'uuid' or key == 'created_at':
                print(f"Cannot update '{key}' field.")
                continue

            if key in backup and backup[key] != value:
                backup[key] = value
                updated = True

        if not updated:
            if print_message:
                print(f"No keys are update for backup with UUID '{backup_uuid}'.")

        backup['updated_at'] = time.time()
        self.save_backup_to_file(backup)
        if print_message and updated:
            print(f"Backup with UUID '{backup_uuid}' updated successfully.")
            
    def check_and_fill_missing_fields(self):
        required_keys = [
            'uuid', 'name', 'source_folder', 'destination_folder',
            'hostname', 'username', 'port', 'key_path', 'key_passphrase',
            'timeout', 'created_at', 'updated_at'
        ]
        
        for backup in self.backups.values():
            missing_keys = [key for key in required_keys if key not in backup]
            if missing_keys:
                for key in missing_keys:
                    if key == 'port':
                        backup[key] = self.DEFAULT_PORT
                    elif key == 'timeout':
                        backup[key] = self.DEFAULT_TIMEOUT
                    elif key in ['created_at', 'updated_at']:
                        backup[key] = time.time()
                    else:
                        backup[key] = None
                        print(f"Backup {backup.get('uuid')} is missing required attribute: {key}")
                        print("It has been set to None. Please update the backup with the correct value.")
                self.save_backup_to_file(backup)
