try:
    import readline
except ImportError:
    import pyreadline3 as readline

from src.backup_info import BackupInfo
from src.ssh_key_manager import SSHKeyManager
from src.command_processor import CommandProcessor
from colorama import Fore

class BackupTool:
    def __init__(self):
        self.backup_info = BackupInfo()
        self.ssh_key_manager = SSHKeyManager()
        self.commands = [
            "list_backups", "add_backup", "update_backup", "delete_backup",
            "list_keys", "create_key", "get_public_key", "delete_key",
            "test_connection", "list_remote_files", "download_files",
            "help", "exit"
        ]
        self.setup_auto_complete()

    def setup_auto_complete(self):
        try:
            readline.parse_and_bind("tab: complete")
            readline.set_completer(self.complete)
        except ImportError:
            print("Tab completion not available. Install readline or pyreadline3 for this feature.")

    def complete(self, text, state):
        options = [cmd for cmd in self.commands if cmd.startswith(text)]
        if state < len(options):
            return options[state]
        return None

    def print_header(self):
        print(f"{Fore.WHITE}Backup Tool - v0.0.1")
        print(f"{Fore.WHITE}A simple backup management tool")

    def print_help(self):
        print(f"{Fore.YELLOW}Available commands:\n")

        print(f"{Fore.GREEN}Backup Commands:")
        print(f"{Fore.WHITE}  list_backups [--full]")
        print(f"{Fore.WHITE}  add_backup <name> <source_folder> <destination_folder> <hostname> <username> [<key_path> <key_passphrase>]")
        print(f"{Fore.WHITE}  update_backup <backup_uuid> [<name> <source_folder> <destination_folder> <hostname> <username> <port> <key_path> <key_passphrase> <timeout>]")
        print(f"{Fore.WHITE}  delete_backup <backup_uuid>")

        print(f"{Fore.GREEN}Key Management Commands:")
        print(f"{Fore.WHITE}  list_keys")
        print(f"{Fore.WHITE}  create_key <key_name> [<key_passphrase>]")
        print(f"{Fore.WHITE}  get_public_key <key_name> [--content]")
        print(f"{Fore.WHITE}  delete_key <key_name>")
        
        print(f"{Fore.GREEN}Backup Connection:")
        print(f"{Fore.WHITE} test_connection <backup_uuid>")
        print(f"{Fore.WHITE} list_remote_files <backup_uuid> [--verify]")
        print(f"{Fore.WHITE} download_files <backup_uuid>")

        print(f"{Fore.GREEN}Other Commands:")
        print(f"{Fore.WHITE}  help")
        print(f"{Fore.WHITE}  exit")

    def run(self):
        self.print_header()
        print()

        try:
            while True:
                command = input("> ").strip().split()
                if not command:
                    continue

                action = command[0].lower()
                args = command[1:]

                if action not in self.commands:
                    print(f"Invalid command: {action}. Type 'help' for available commands.")
                    continue

                if action == "help":
                    self.print_help()

                elif action == "exit":
                    print("Exiting...")
                    break

                else:
                    processor = CommandProcessor(self.backup_info, self.ssh_key_manager)
                    processor.process_command(action, args)

        except EOFError:
            print("\nExiting...")
        except KeyboardInterrupt:
            print("\nKeyboard interrupt detected. Exiting...")
        except Exception as e:
            print(f"An unexpected error occurred: {e}")
