import os
import getpass
from tabulate import tabulate
from src.ssh_connector import SSHConnector
from src.file_manager import FileManager
from src.download_manager import DownloadManager

class CommandProcessor:
    def __init__(self, backup_info, ssh_key_manager):
        self.backup_info = backup_info
        self.ssh_key_manager = ssh_key_manager
        self.command_map = {
            "add_backup": self._add_backup,
            "list_backups": self._list_backups,
            "update_backup": self._update_backup,
            "delete_backup": self._delete_backup,
            "create_key": self._create_key,
            "delete_key": self._delete_key,
            "get_public_key": self._get_public_key,
            "list_keys": self._list_keys,
            "test_connection": self._test_connection,
            "list_remote_files": self._list_remote_files,
            "download_files": self._download_files
        }

    def process_command(self, action, args):
        command = self.command_map.get(action)
        if command:
            command(args)
        else:
            print("Invalid command. Type 'help' for available commands.")

    def _print_usage(self, command, invalid_number_of_arguments=False):
        if invalid_number_of_arguments:
            print("Invalid number of arguments.")

        usage_map = {
            "add_backup": "Usage: add_backup <name> <source_folder> <destination_folder> <hostname> <username> [<key_path> <key_passphrase>]",
            "update_backup": "Usage: update_backup <backup_uuid> [<name> <source_folder> <destination_folder> <hostname> <username> <port> <key_path> <key_passphrase> <timeout>]",
            "delete_backup": "Usage: delete_backup <backup_uuid>",
            "create_key": "Usage: create_key <key_name> [<key_passphrase>]",
            "delete_key": "Usage: delete_key <key_name>",
            "get_public_key": "Usage: get_public_key <key_name> [--content]",
            "test_connection": "Usage: test_connection <backup_uuid>",
            "list_remote_files": "Usage: list_remote_files <backup_uuid> [--verify]",
            "download_files": "Usage: download_files <backup_uuid>"
        }
        print(usage_map.get(command, "Invalid command"))

    def _add_backup(self, args):
        if len(args) < 5:
            return self._print_usage("add_backup", True)

        name, source_folder, destination_folder, hostname, username = args[:5]
        key_path = args[5] if len(args) > 5 else None
        key_passphrase = args[6] if len(args) > 6 else None

        backup_details = {
            'name': name,
            'source_folder': source_folder,
            'destination_folder': destination_folder,
            'hostname': hostname,
            'username': username,
            'key_path': key_path,
            'key_passphrase': key_passphrase,
        }

        try:
            backup = self.backup_info.add_backup(backup_details)
        except ValueError as e:
            print(f"Error: {e}")
            return self._print_usage("add_backup")

        if not key_path:
            self._handle_ssh_key_creation(backup)

        self._print_backup_details(backup['uuid'])
       
    def _handle_ssh_key_creation(self, backup):
        if self._prompt_yes_no("Do you want to create an SSH key for this backup? (y/N): "):
            key_passphrase = None
            if self._prompt_yes_no("Do you want to set a passphrase for the SSH key? (Y/n): ", default_yes=True):
                key_passphrase = getpass.getpass("Enter the passphrase for the SSH key: ")

            self.ssh_key_manager.generate_keys(backup['uuid'], key_passphrase)
            key_path = self.ssh_key_manager.get_private_key(backup['uuid'])
            self.backup_info.update_backup(backup['uuid'], print_message=False, key_path=key_path, key_passphrase=key_passphrase)

    def _print_backup_details(self, backup_uuid):
        backup = self.backup_info.load_backups_from_files().get(backup_uuid, {})
        headers = ["UUID", "Name", "Source Folder", "Destination Folder", "Hostname", "Username", "Key Path"]
        row = (backup['uuid'], backup['name'], backup['source_folder'], backup['destination_folder'], backup.get("hostname", ""), backup.get("username", ""), backup.get("key_path", ""))
        print(tabulate([row], headers=headers, tablefmt="fancy_grid"))

    def _list_backups(self, args):
        if args and args[0] == "--full":
            self._list_backups_full()
        else:
            self._list_backups_simple()

    def _list_backups_simple(self):
        print("If you want to see more details, type 'list_backups --full'.")
        headers = ["UUID", "Name"]
        rows = [(uuid, backup['name']) for uuid, backup in self.backup_info.backups.items()]
        print(tabulate(rows, headers=headers, tablefmt="fancy_grid"))

    def _list_backups_full(self):
        headers = ["UUID", "Name", "Source Folder", "Destination Folder", "Hostname", "Port", "Username", "Key Path"]
        rows = [(uuid, backup['name'], backup['source_folder'], backup['destination_folder'], backup.get("hostname", ""), backup.get("port", ""), backup.get("username", ""), backup.get("key_path", "")) for uuid, backup in self.backup_info.backups.items()]
        print(tabulate(rows, headers=headers, tablefmt="fancy_grid"))

    def _update_backup(self, args):
        if len(args) < 1:
            return self._print_usage("update_backup")

        backup_uuid = args[0]
        if len(args[1:]) % 2 != 0:
            print("Error: Arguments must be in key-value pairs.")
            return

        kwargs = {args[i]: args[i + 1] for i in range(1, len(args), 2)}
        self.backup_info.update_backup(backup_uuid, **kwargs)

    def _delete_backup(self, args):
        if len(args) < 1:
            return self._print_usage("delete_backup")

        backup_uuid = args[0]
        try:
            self.backup_info.delete_backup(backup_uuid)
        except Exception as e:
            print(f"Error: {e}")

    def _create_key(self, args):
        if len(args) < 1:
            return self._print_usage("create_key")

        password = args[1] if len(args) > 1 else None
        self.ssh_key_manager.generate_keys(args[0], password)
        
    def _delete_key(self, args):
        if len(args) < 1:
            return self._print_usage("delete_key")

        self.ssh_key_manager.delete_key(args[0])

    def _get_public_key(self, args):
        if len(args) < 1:
            return self._print_usage("get_public_key")

        if len(args) > 1 and args[1] == "--content":
            self._get_public_key_content(args[0])
        else:
            self._get_public_key_path(args[0])

    def _get_public_key_path(self, key_name):
        print("If you want to see the key content, type get_public_key <key_name> [--content].")
        print(self.ssh_key_manager.get_public_key(key_name))
        
    def _get_public_key_content(self, key_name):
        with open(self.ssh_key_manager.get_public_key(key_name), 'r') as f:
            print(f.read())
            
    def _list_keys(self, _):
        keys = self.ssh_key_manager.list_keys()
        if not keys:
            print("No keys found.")
            return

        print(tabulate(keys, headers="keys", tablefmt="fancy_grid"))

    def _test_connection(self, args):
        if len(args) < 1:
            return self._print_usage("test_connection")

        backups = self.backup_info.load_backups_from_files()
        backup_uuid = args[0]

        if backup_uuid not in backups:
            print("Backup not found.")
            return

        backup = backups[backup_uuid]
        print(f"Testing connection for backup: {backup['name']}")

        ssh_connector = self._get_ssh_connector(backup)
        ssh_connector.connect()
        ssh_connector.disconnect()

    def _list_remote_files(self, args):
        if len(args) < 1:
            return self._print_usage("list_remote_files")

        backup_uuid = args[0]
        verify = len(args) > 1 and args[1] == "--verify"
        backup = self.backup_info.load_backups_from_files().get(backup_uuid)

        if not backup:
            print("Backup not found.")
            return

        ssh_connector = self._get_ssh_connector(backup)
        if ssh_connector.connect():
            file_list = ssh_connector.execute_command(f"ls {backup['source_folder']}")
            file_manager_result = FileManager.list_files(file_list)

            if verify:
                file_manager_result = FileManager.verify_files(ssh_connector, backup, file_manager_result)

                if not file_manager_result:
                    ssh_connector.disconnect()
                    return

            self._display_remote_files(file_manager_result)
            ssh_connector.disconnect()

    def _display_remote_files(self, file_manager_result):
        if file_manager_result['group_count'] == 0:
            print("No files found on the remote server.")
        
        if 'verify_groups' in file_manager_result and file_manager_result['verify_groups']:
            headers = ["Name", "Sha256", "Status", "Is ready to download?"]
            print(tabulate(file_manager_result['verify_groups'], headers=headers, tablefmt="fancy_grid"))
        else:
            print('Check the status of the files on the remote server with --verify option.')
            headers = ["Name", "Sha256", "Status"]
            print(tabulate(file_manager_result['groups'], headers=headers, tablefmt="fancy_grid"))

    def _download_files(self, args):
        if len(args) < 1:
            return self._print_usage("download_files")

        backup_uuid = args[0]
        backups = self.backup_info.load_backups_from_files()

        if backup_uuid not in backups:
            print("Backup not found.")
            return

        backup = backups[backup_uuid]
        ssh_connector = self._get_ssh_connector(backup)
        
        try:
            if ssh_connector.connect():
                file_list = ssh_connector.execute_command(f"ls {backup['source_folder']}")
                file_manager_result = FileManager.list_files(file_list)
                file_manager_result_verified = FileManager.verify_files(ssh_connector, backup, file_manager_result)

                if not file_manager_result_verified:
                    ssh_connector.disconnect()
                    return

                for group in file_manager_result_verified['verify_groups']:
                    if group[3]:
                        if FileManager.check_for_existing_local_files(backup['destination_folder'], group[:2]):
                            print(f"Files {group[0]} and {group[1]} already exist in the destination folder.")
                            continue

                        download_manager = DownloadManager(ssh_connector)
                        download_manager.set_status("downloading", backup['source_folder'], group[2])

                        downloaded_files = []
                        failed = False
                        for file in group[:2]:
                            if download_manager.download_file(f"{backup['source_folder']}/{file}", backup['destination_folder'], file):
                                downloaded_files.append(file)
                            else:
                                failed = True
                                for downloaded_file in downloaded_files:
                                    os.remove(os.path.join(backup['destination_folder'], downloaded_file))

                        if not failed and download_manager.verify_sha256(os.path.join(backup['destination_folder'], group[0]), os.path.join(backup['destination_folder'], group[1])):
                            download_manager.set_status("downloaded", backup['source_folder'], group[2])
                        else:
                            for file in group[:2]:
                                os.remove(os.path.join(backup['destination_folder'], file))
                            download_manager.set_status("failed", backup['source_folder'], group[2])
                
                ssh_connector.disconnect()
    
        except Exception as e:
            print(f"Error occurred during download: {e}")
            ssh_connector.disconnect()

    def _prompt_yes_no(self, message, default_yes=False):
        response = input(message).strip().lower()
        if default_yes:
            return response not in ("n", "no")
        else:
            return response in ("y", "yes")
        
    def _get_ssh_connector(self, backup):
        return SSHConnector(
            backup['hostname'],
            backup.get('port', 22),
            backup['username'],
            backup['key_path'],
            backup['key_passphrase'],
            backup.get('timeout', 30)
        )
