import paramiko
import os
import time

class SSHConnector:
    def __init__(self, hostname, port=22, username=None, ssh_key_path=None, ssh_key_pass=None, timeout=5):
        self.hostname = hostname
        self.port = port
        self.username = username
        self.ssh_key_path = ssh_key_path
        self.ssh_key_pass = ssh_key_pass
        self.timeout = timeout
        self.ssh_client = None

    def connect(self):
        retries = 3
        for attempt in range(retries):
            try:
                self.ssh_client = paramiko.SSHClient()
                self.ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

                if self.ssh_key_path:
                    private_key = self._load_private_key()
                    if private_key is None:
                        raise RuntimeError(f"Failed to load private key {self.ssh_key_path}.")
                else:
                    private_key = None

                self.ssh_client.connect(self.hostname, port=self.port, username=self.username, pkey=private_key, timeout=self.timeout)
                print("SSH connection established.")
                return True
            except Exception as e:
                print(f"Failed to connect to {self.hostname}: {e}")
                if attempt < retries - 1:
                    print(f"Retrying SSH connection attempt {attempt + 2}/{retries} after 5 seconds...")
                    time.sleep(5)
                else:
                    return False

    def execute_command(self, command):
        try:
            stdin, stdout, stderr = self.ssh_client.exec_command(command)
            return stdout.read().decode().strip()
        except Exception as e:
            print(f"Error executing command '{command}': {e}")
            return ''

    def download(self, remote_path, local_path):
        try:
            with paramiko.SFTPClient.from_transport(self.ssh_client.get_transport()) as sftp:
                sftp.get(remote_path, local_path)
            return True
        except Exception as e:
            print(f"Error downloading file '{remote_path}' to '{local_path}': {e}")
            if os.path.exists(local_path):
                os.remove(local_path)
            return False

    def disconnect(self):
        if self.ssh_client:
            self.ssh_client.close()
            print("SSH connection closed.")

    def _load_private_key(self):
        try:
            if self.ssh_key_pass:
                private_key = paramiko.RSAKey.from_private_key_file(self.ssh_key_path, password=self.ssh_key_pass)
            else:
                private_key = paramiko.RSAKey.from_private_key_file(self.ssh_key_path)
            return private_key
        except Exception as e:
            print(f"Error loading private key {self.ssh_key_path}: {e}")
            return None
