import os
import paramiko

class SSHKeyManager:
    def __init__(self, key_folder="keys/"):
        self.key_folder = key_folder
        os.makedirs(self.key_folder, exist_ok=True)

    def generate_keys(self, key_name, key_passphrase=None):
        private_key_path = os.path.join(self.key_folder, f"{key_name}_private.pem")
        public_key_path = os.path.join(self.key_folder, f"{key_name}_public.pem")

        if os.path.exists(private_key_path):
            print(f"Keys with name '{key_name}' already exist.")
            return

        key = paramiko.RSAKey.generate(2048)
        key.write_private_key_file(private_key_path, password=key_passphrase)
        with open(public_key_path, 'w') as f:
            f.write(f"{key.get_name()} {key.get_base64()}")
        print(f"Keys '{key_name}' generated successfully.")

    def list_keys(self):
        key_files = os.listdir(self.key_folder)
        key_dict = {}

        for file_name in key_files:
            if file_name.endswith("_private.pem"):
                name = file_name.replace("_private.pem", "")
                key_dict[name] = key_dict.get(name, {"name": name, "private_key": False, "public_key": False})
                key_dict[name]["private_key"] = True
            elif file_name.endswith("_public.pem"):
                name = file_name.replace("_public.pem", "")
                key_dict[name] = key_dict.get(name, {"name": name, "private_key": False, "public_key": False})
                key_dict[name]["public_key"] = True

        return list(key_dict.values())

    def get_private_key(self, key_name):
        key_file = os.path.join(self.key_folder, f"{key_name}_private.pem")
        if not os.path.exists(key_file):
            print(f"Private key '{key_name}' not found.")
            return None
        return key_file

    def get_public_key(self, key_name):
        key_file = os.path.join(self.key_folder, f"{key_name}_public.pem")
        if not os.path.exists(key_file):
            print(f"Public key '{key_name}' not found.")
            return None
        return key_file
    
    def delete_key(self, key_name):
        private_key_file = os.path.join(self.key_folder, f"{key_name}_private.pem")
        public_key_file = os.path.join(self.key_folder, f"{key_name}_public.pem")

        if os.path.exists(private_key_file):
            os.remove(private_key_file)
            print(f"Private key '{key_name}' deleted.")
        else:
            print(f"No private key found with name '{key_name}'.")

        if os.path.exists(public_key_file):
            os.remove(public_key_file)
            print(f"Public key '{key_name}' deleted.")
        else:
            print(f"No public key found with name '{key_name}'.")
